package com.nisum.learning.student.client;

import com.nisum.learning.student.model.Student;
import org.springframework.web.reactive.function.client.WebClient;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

public class StudentWebClient {
    WebClient webClient = WebClient.builder().baseUrl("http://localhost:8080").build();

    public Flux<Student> findAll(){
        Flux<Student> students = webClient.get()
                .uri("/students")
                .exchangeToFlux(clientResponse -> clientResponse.bodyToFlux(Student.class));
        return students;
    }

    public Mono<Student> create(Student newStudent) {
        Mono<Student> student = webClient.post()
                .uri("/students")
                .body(Mono.just(newStudent), Student.class)
                .retrieve()
                .bodyToMono(Student.class);
        return student;
    }

    public Mono<Student> update(Student s) {
        Mono<Student> student = webClient.put()
                .uri(uriBuilder -> uriBuilder.path("/students/{id}").build(s.getId()))
                .body(Mono.just(s), Student.class)
                .retrieve()
                .bodyToMono(Student.class);
        return student;
    }

    public void delete(Integer id) {
        webClient.delete()
                .uri(uriBuilder -> uriBuilder.path("/students/{id}").build(id))
                .retrieve();
    }
}
