package com.nisum.learning.student.controller;

import com.nisum.learning.student.model.Student;
import com.nisum.learning.student.service.StudentService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.*;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

@RestController
@RequestMapping("/students")
public class StudentController {
    @Autowired
    StudentService studentService;

    //for demo/ref purposes
    public static void main(String[] args) {
        Flux<Integer> nums = Flux.just(1, 2, 3)/*.concatWith(Flux.error(new RuntimeException("Exception occurred")))*/;
        nums.log()
                .subscribe(System.out::println,
                        throwable -> System.out.println("Error: " + throwable.getMessage()),
                        () -> System.out.println("successful"));
    }

    @GetMapping()
    public Flux<Student> getAllStudents(){
        return studentService.findAll();
    }

    @GetMapping("/{id}")
    public Mono<Student> getStudentById(@PathVariable int id){
        return studentService.findById(id);
    }

    @PostMapping()
    @ResponseStatus(HttpStatus.CREATED)
    public void createStudent(@RequestBody Student newStudent){
        studentService.create(newStudent);
    }

    @PutMapping("/{id}")
    public Mono<Student> updateStudentById(@RequestBody Student newStudent){
        return studentService.update(newStudent);
    }

    @DeleteMapping("/{id}")
    public void deleteStudentById(@PathVariable int id){
        studentService.deleteById(id);
    }
}
